# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: kernel.proto
# plugin: python-betterproto
# This file has been @generated

from dataclasses import dataclass

import betterproto


@dataclass(eq=False, repr=False)
class AccountAddress(betterproto.Message):
    """An address of an account. Always 32 bytes."""

    value: bytes = betterproto.bytes_field(1)


@dataclass(eq=False, repr=False)
class Memo(betterproto.Message):
    """
    A memo which can be included as part of a transfer. Max size is 256 bytes.
    This message can occur starting from protocol version 2.
    """

    value: bytes = betterproto.bytes_field(1)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: protocol-level-tokens.proto
# Protobuf Python Version: 4.25.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import ccdexplorer_fundamentals.GRPCClient.kernel_pb2 as kernel__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x1bprotocol-level-tokens.proto\x12\x11\x63oncordium.v2.plt\x1a\x0ckernel.proto"\x15\n\x04\x43\x62or\x12\r\n\x05value\x18\x01 \x01(\x0c"\x18\n\x07TokenId\x12\r\n\x05value\x18\x01 \x01(\t"\x1f\n\x0eTokenModuleRef\x12\r\n\x05value\x18\x01 \x01(\x0c".\n\x0bTokenAmount\x12\r\n\x05value\x18\x01 \x01(\x04\x12\x10\n\x08\x64\x65\x63imals\x18\x02 \x01(\r"\xc6\x01\n\nTokenState\x12;\n\x10token_module_ref\x18\x01 \x01(\x0b\x32!.concordium.v2.plt.TokenModuleRef\x12\x10\n\x08\x64\x65\x63imals\x18\x03 \x01(\r\x12\x34\n\x0ctotal_supply\x18\x04 \x01(\x0b\x32\x1e.concordium.v2.plt.TokenAmount\x12-\n\x0cmodule_state\x18\x05 \x01(\x0b\x32\x17.concordium.v2.plt.CborJ\x04\x08\x02\x10\x03"\x95\x01\n\x11TokenAccountState\x12/\n\x07\x62\x61lance\x18\x01 \x01(\x0b\x32\x1e.concordium.v2.plt.TokenAmount\x12\x32\n\x0cmodule_state\x18\x04 \x01(\x0b\x32\x17.concordium.v2.plt.CborH\x00\x88\x01\x01\x42\x0f\n\r_module_stateJ\x04\x08\x02\x10\x03J\x04\x08\x03\x10\x04"J\n\x10TokenModuleEvent\x12\x0c\n\x04type\x18\x01 \x01(\t\x12(\n\x07\x64\x65tails\x18\x02 \x01(\x0b\x32\x17.concordium.v2.plt.Cbor"J\n\x0bTokenHolder\x12\x30\n\x07\x61\x63\x63ount\x18\x01 \x01(\x0b\x32\x1d.concordium.v2.AccountAddressH\x00\x42\t\n\x07\x61\x64\x64ress"\xcf\x01\n\x12TokenTransferEvent\x12,\n\x04\x66rom\x18\x01 \x01(\x0b\x32\x1e.concordium.v2.plt.TokenHolder\x12*\n\x02to\x18\x02 \x01(\x0b\x32\x1e.concordium.v2.plt.TokenHolder\x12.\n\x06\x61mount\x18\x03 \x01(\x0b\x32\x1e.concordium.v2.plt.TokenAmount\x12&\n\x04memo\x18\x04 \x01(\x0b\x32\x13.concordium.v2.MemoH\x00\x88\x01\x01\x42\x07\n\x05_memo"x\n\x16TokenSupplyUpdateEvent\x12.\n\x06target\x18\x01 \x01(\x0b\x32\x1e.concordium.v2.plt.TokenHolder\x12.\n\x06\x61mount\x18\x02 \x01(\x0b\x32\x1e.concordium.v2.plt.TokenAmount"\xc3\x02\n\nTokenEvent\x12,\n\x08token_id\x18\x01 \x01(\x0b\x32\x1a.concordium.v2.plt.TokenId\x12;\n\x0cmodule_event\x18\x02 \x01(\x0b\x32#.concordium.v2.plt.TokenModuleEventH\x00\x12?\n\x0etransfer_event\x18\x03 \x01(\x0b\x32%.concordium.v2.plt.TokenTransferEventH\x00\x12?\n\nmint_event\x18\x04 \x01(\x0b\x32).concordium.v2.plt.TokenSupplyUpdateEventH\x00\x12?\n\nburn_event\x18\x05 \x01(\x0b\x32).concordium.v2.plt.TokenSupplyUpdateEventH\x00\x42\x07\n\x05\x65vent"<\n\x0bTokenEffect\x12-\n\x06\x65vents\x18\x01 \x03(\x0b\x32\x1d.concordium.v2.plt.TokenEvent"\x90\x01\n\x17TokenModuleRejectReason\x12,\n\x08token_id\x18\x01 \x01(\x0b\x32\x1a.concordium.v2.plt.TokenId\x12\x0c\n\x04type\x18\x02 \x01(\t\x12-\n\x07\x64\x65tails\x18\x03 \x01(\x0b\x32\x17.concordium.v2.plt.CborH\x00\x88\x01\x01\x42\n\n\x08_details"\xc6\x01\n\tCreatePLT\x12,\n\x08token_id\x18\x01 \x01(\x0b\x32\x1a.concordium.v2.plt.TokenId\x12\x37\n\x0ctoken_module\x18\x02 \x01(\x0b\x32!.concordium.v2.plt.TokenModuleRef\x12\x10\n\x08\x64\x65\x63imals\x18\x04 \x01(\r\x12:\n\x19initialization_parameters\x18\x05 \x01(\x0b\x32\x17.concordium.v2.plt.CborJ\x04\x08\x03\x10\x04"w\n\x14TokenCreationDetails\x12\x30\n\ncreate_plt\x18\x01 \x01(\x0b\x32\x1c.concordium.v2.plt.CreatePLT\x12-\n\x06\x65vents\x18\x02 \x03(\x0b\x32\x1d.concordium.v2.plt.TokenEventB=\n\x1a\x63om.concordium.grpc.v2.pltP\x01Z\x04./pb\xaa\x02\x16\x43oncordium.Grpc.V2.Pltb\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(
    DESCRIPTOR, "protocol_level_tokens_pb2", _globals
)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals["DESCRIPTOR"]._serialized_options = (
        b"\n\032com.concordium.grpc.v2.pltP\001Z\004./pb\252\002\026Concordium.Grpc.V2.Plt"
    )
    _globals["_CBOR"]._serialized_start = 64
    _globals["_CBOR"]._serialized_end = 85
    _globals["_TOKENID"]._serialized_start = 87
    _globals["_TOKENID"]._serialized_end = 111
    _globals["_TOKENMODULEREF"]._serialized_start = 113
    _globals["_TOKENMODULEREF"]._serialized_end = 144
    _globals["_TOKENAMOUNT"]._serialized_start = 146
    _globals["_TOKENAMOUNT"]._serialized_end = 192
    _globals["_TOKENSTATE"]._serialized_start = 195
    _globals["_TOKENSTATE"]._serialized_end = 393
    _globals["_TOKENACCOUNTSTATE"]._serialized_start = 396
    _globals["_TOKENACCOUNTSTATE"]._serialized_end = 545
    _globals["_TOKENMODULEEVENT"]._serialized_start = 547
    _globals["_TOKENMODULEEVENT"]._serialized_end = 621
    _globals["_TOKENHOLDER"]._serialized_start = 623
    _globals["_TOKENHOLDER"]._serialized_end = 697
    _globals["_TOKENTRANSFEREVENT"]._serialized_start = 700
    _globals["_TOKENTRANSFEREVENT"]._serialized_end = 907
    _globals["_TOKENSUPPLYUPDATEEVENT"]._serialized_start = 909
    _globals["_TOKENSUPPLYUPDATEEVENT"]._serialized_end = 1029
    _globals["_TOKENEVENT"]._serialized_start = 1032
    _globals["_TOKENEVENT"]._serialized_end = 1355
    _globals["_TOKENEFFECT"]._serialized_start = 1357
    _globals["_TOKENEFFECT"]._serialized_end = 1417
    _globals["_TOKENMODULEREJECTREASON"]._serialized_start = 1420
    _globals["_TOKENMODULEREJECTREASON"]._serialized_end = 1564
    _globals["_CREATEPLT"]._serialized_start = 1567
    _globals["_CREATEPLT"]._serialized_end = 1765
    _globals["_TOKENCREATIONDETAILS"]._serialized_start = 1767
    _globals["_TOKENCREATIONDETAILS"]._serialized_end = 1886
# @@protoc_insertion_point(module_scope)



from pydantic import BaseModel


class BakerStakePendingChange(BaseModel):
    pendingChangeType: str


class CurrentPaydayStatus(BaseModel):
    bakerEquityCapital: str
    blocksBaked: int
    delegatedCapital: str
    effectiveStake: str
    finalizationLive: bool
    lotteryPower: float
    transactionFeesEarned: str


class CommissionRates(BaseModel):
    bakingCommission: float
    finalizationCommission: float
    transactionCommission: float


class PoolInfo(BaseModel):
    commissionRates: CommissionRates
    metadataUrl: str
    openStatus: str


class APY(BaseModel):
    bakerApy: float
    delegatorsApy: float = None
    totalApy: float


class Rank(BaseModel):
    rank: int
    total: int


class ConcordiumPoolFromClient(BaseModel):
    allPoolTotalCapital: str
    bakerAddress: str
    bakerEquityCapital: str
    bakerId: int
    bakerStakePendingChange: BakerStakePendingChange
    currentPaydayStatus: CurrentPaydayStatus
    delegatedCapital: str
    delegatedCapitalCap: str
    poolInfo: PoolInfo
    poolType: str


class ConcordiumPoolFromCCDScan(BaseModel):
    apy_30: APY
    apy_7: APY
    openStatus: str
    metadataUrl: str
    delegatorCount: int
    delegatedStake: int
    delegatedStakeCap: int
    rankingByTotalStake: Rank
    totalStake: int
    totalStakePercentage: float
    commissionRates: CommissionRates


class PeriodReward(BaseModel):
    sumBakerRewardAmount: int
    sumDelegatorsRewardAmount: int
    sumTotalRewardAmount: int


class ConcordiumPoolRewards(BaseModel):
    LAST24_HOURS: PeriodReward
    LAST7_DAYS: PeriodReward
    LAST30_DAYS: PeriodReward


class ConcordiumPool(BaseModel):
    via_client: ConcordiumPoolFromClient = None
    via_ccdscan: ConcordiumPoolFromCCDScan = None
    rewards: ConcordiumPoolRewards = None



import pymongo
from pymongo import DESCENDING


class Mixin:
    def search_txs_on_list_of_hashes(
        self, list_of_hashses, skip, limit, sort_on="amount", sort_direction=-1
    ):
        """
        This pipeline is built for the collection 'transactions'.
        """
        sort_direction = (
            pymongo.ASCENDING if sort_direction == 1 else pymongo.DESCENDING
        )
        sort_condition = (
            {"$sort": {"amount_ccd": sort_direction}}
            if sort_on == "amount"
            else {"$sort": {"blockHeight": sort_direction}}
        )
        pipeline = [
            {"$match": {"_id": {"$in": list_of_hashses}}},
            {
                "$addFields": {
                    "amount_ccd": {
                        "$first": {
                            "$map": {
                                "input": "$result.events",
                                "as": "events",
                                "in": {
                                    "$trunc": {
                                        "$divide": [
                                            {"$toDouble": "$$events.amount"},
                                            1000000,
                                        ]
                                    }
                                },
                            }
                        }
                    }
                }
            },
            {
                "$facet": {
                    "metadata": [{"$count": "total"}],
                    "data": [{"$skip": int(skip)}, {"$limit": int(limit)}],
                }
            },
            {
                "$project": {
                    "data": 1,
                    "total": {"$arrayElemAt": ["$metadata.total", 0]},
                }
            },
        ]

        return pipeline

    def search_transfers_based_on_indices(
        self,
        transfer_type,
        gte,
        lte,
        start_block,
        end_block,
        # skip, limit,
        list_of_tx_hashes_with_memo_predicate,
        # sort_on='amount', sort_direction=-1,
        filter_on_memo_txs=False,
    ):
        pipeline = []

        if transfer_type == "scheduled":
            pipeline.extend(
                [{"$match": {"contents": {"$in": ["transferWithSchedule"]}}}]
            )

        if filter_on_memo_txs:
            pipeline.extend(
                [{"$match": {"_id": {"$in": list_of_tx_hashes_with_memo_predicate}}}]
            )

        pipeline.extend(
            [
                {"$match": {"amount": {"$gte": gte}}},
                {"$match": {"amount": {"$lte": lte}}},
                {"$match": {"blockHeight": {"$gt": start_block, "$lte": end_block}}},
                # { '$facet':     { 'metadata': [ { '$count': 'total' } ],
                #                 'data': [ { '$skip': int(skip) }, { '$limit': int(limit) } ]
                #                 }
                # }
            ]
        )

        return pipeline

    def search_transfers_based_on_indices_v2(
        self,
        transfer_type,
        gte,
        lte,
        start_block,
        end_block,
        # skip, limit,
        list_of_tx_hashes_with_memo_predicate,
        # sort_on='amount', sort_direction=-1,
        filter_on_memo_txs=False,
    ):
        pipeline = []

        if transfer_type == "transferred_with_schedule":
            pipeline.extend(
                [{"$match": {"contents": {"$in": ["transferred_with_schedule"]}}}]
            )

        if filter_on_memo_txs:
            pipeline.extend(
                [{"$match": {"_id": {"$in": list_of_tx_hashes_with_memo_predicate}}}]
            )

        pipeline.extend(
            [
                {"$set": {"amount_ccd": {"$divide": ["$amount", 1000000]}}},
                {"$match": {"amount_ccd": {"$gte": gte}}},
                {"$match": {"amount_ccd": {"$lte": lte}}},
                {"$match": {"block_height": {"$gt": start_block, "$lte": end_block}}},
            ]
        )

        return pipeline

    def search_transfers_mongo_v2(
        self,
        transfer_type,
        gte,
        lte,
        start_date,
        end_date,
        skip,
        limit,
        list_of_tx_hashes,
        sort_on="amount",
        sort_direction=-1,
        filter_on_tx_hashes=False,
    ):
        """
        This pipeline now includes all transfers, so regular, scheduled and with memo.
        """

        sort_direction = (
            pymongo.ASCENDING if sort_direction == 1 else pymongo.DESCENDING
        )
        sort_condition = (
            {"$sort": {"amount_ccd": sort_direction}}
            if sort_on == "amount"
            else {"$sort": {"block_info.slot_time": sort_direction}}
        )
        type_contents_condition = [transfer_type]
        pipeline = [
            {"$match": {"type.contents": {"$in": type_contents_condition}}},
        ]
        if filter_on_tx_hashes:
            pipeline.extend([{"$match": {"_id": {"$in": list_of_tx_hashes}}}])

        if transfer_type == "transferred_with_schedule":
            pipeline.extend(
                [
                    {
                        "$set": {
                            "amount_ccd": {
                                "$sum": {
                                    "$map": {
                                        "input": "$account_transaction.effects.transferred_with_schedule.amount",
                                        "as": "event",
                                        "in": {"$divide": ["$$event.amount", 1000000]},
                                    }
                                }
                            }
                        }
                    }
                ]
            )
        else:
            # so regular transfer
            pipeline.extend(
                [
                    {
                        "$set": {
                            "amount_ccd": {
                                "$divide": [
                                    "$account_transaction.effects.account_transfer.amount",
                                    1000000,
                                ]
                            }
                        }
                    }
                ]
            )

        pipeline.extend(
            [
                {"$match": {"amount_ccd": {"$gte": gte}}},
                {"$match": {"amount_ccd": {"$lte": lte}}},
                {
                    "$match": {
                        "block_info.slot_time": {"$gte": start_date, "$lt": end_date}
                    }
                },
                sort_condition,
                {
                    "$facet": {
                        "metadata": [{"$count": "total"}],
                        "data": [{"$skip": int(skip)}, {"$limit": int(limit)}],
                    }
                },
                {
                    "$project": {
                        "data": 1,
                        "total": {"$arrayElemAt": ["$metadata.total", 0]},
                    }
                },
            ]
        )

        return pipeline

    def search_scheduled_transfers_mongo_v2(
        self,
        transfer_type,
        gte,
        lte,
        start_date,
        end_date,
        skip,
        limit,
        list_of_tx_hashes,
        sort_on="amount",
        sort_direction=-1,
        filter_on_tx_hashes=False,
    ):
        """
        This pipeline now includes all transfers, so regular, scheduled and with memo.
        """

        sort_direction = (
            pymongo.ASCENDING if sort_direction == 1 else pymongo.DESCENDING
        )
        sort_condition = (
            {"$sort": {"amount_ccd": sort_direction}}
            if sort_on == "amount"
            else {"$sort": {"block_info.slot_time": sort_direction}}
        )
        type_contents_condition = ["transferred_with_schedule"]
        pipeline = [
            {"$match": {"type.contents": {"$in": type_contents_condition}}},
        ]
        if filter_on_tx_hashes:
            pipeline.extend([{"$match": {"_id": {"$in": list_of_tx_hashes}}}])

        pipeline.extend(
            [
                {
                    "$set": {
                        "amount_ccd": {
                            "$sum": {
                                "$map": {
                                    "input": "$account_transaction.effects.transferred_with_schedule.amount",
                                    "as": "event",
                                    "in": {"$divide": ["$$event.amount", 1000000]},
                                }
                            }
                        }
                    }
                },
                {"$match": {"amount_ccd": {"$gte": gte}}},
                {"$match": {"amount_ccd": {"$lte": lte}}},
                {
                    "$match": {
                        "block_info.slot_time": {"$gte": start_date, "$lt": end_date}
                    }
                },
                sort_condition,
                {
                    "$facet": {
                        "metadata": [{"$count": "total"}],
                        "data": [{"$skip": int(skip)}, {"$limit": int(limit)}],
                    }
                },
                {
                    "$project": {
                        "data": 1,
                        "total": {"$arrayElemAt": ["$metadata.total", 0]},
                    }
                },
            ]
        )

        return pipeline

    def search_scheduled_transfers_mongo(
        self,
        transfer_type,
        gte,
        lte,
        start_date,
        end_date,
        skip,
        limit,
        list_of_tx_hashes,
        sort_on="amount",
        sort_direction=-1,
        filter_on_tx_hashes=False,
    ):
        sort_direction = (
            pymongo.ASCENDING if sort_direction == 1 else pymongo.DESCENDING
        )
        sort_condition = (
            {"$sort": {"amount_ccd": sort_direction}}
            if sort_on == "amount"
            else {"$sort": {"blockInfo.blockSlotTime": sort_direction}}
        )
        type_contents_condition = ["transferWithSchedule"]
        pipeline = [
            {"$match": {"type.contents": {"$in": type_contents_condition}}},
        ]
        if filter_on_tx_hashes:
            pipeline.extend([{"$match": {"_id": {"$in": list_of_tx_hashes}}}])

        pipeline.extend(
            [
                {
                    "$set": {
                        "amount_ccd": {
                            "$sum": {
                                "$first": {
                                    "$map": {
                                        "input": "$result.events",
                                        "as": "event",
                                        "in": {
                                            "$map": {
                                                "input": "$$event.amount",
                                                "as": "amount",
                                                "in": {
                                                    "$divide": [
                                                        {
                                                            "$toDouble": {
                                                                "$last": "$$amount"
                                                            }
                                                        },
                                                        1000000,
                                                    ]
                                                },
                                            }
                                        },
                                    }
                                }
                            }
                        }
                    }
                },
                {"$match": {"amount_ccd": {"$gte": gte}}},
                {"$match": {"amount_ccd": {"$lte": lte}}},
                {"$match": {"blockSlotTime": {"$gte": start_date, "$lt": end_date}}},
                sort_condition,
                {
                    "$facet": {
                        "metadata": [{"$count": "total"}],
                        "data": [{"$skip": int(skip)}, {"$limit": int(limit)}],
                    }
                },
                {
                    "$project": {
                        "data": 1,
                        "total": {"$arrayElemAt": ["$metadata.total", 0]},
                    }
                },
            ]
        )

        return pipeline

    def search_transfers_mongo(
        self,
        transfer_type,
        gte,
        lte,
        start_date,
        end_date,
        skip,
        limit,
        list_of_tx_hashes,
        sort_on="amount",
        sort_direction=-1,
        filter_on_tx_hashes=False,
    ):
        sort_direction = (
            pymongo.ASCENDING if sort_direction == 1 else pymongo.DESCENDING
        )
        sort_condition = (
            {"$sort": {"amount_ccd": sort_direction}}
            if sort_on == "amount"
            else {"$sort": {"blockInfo.blockSlotTime": sort_direction}}
        )
        type_contents_condition = ["transfer", "transferWithMemo"]

        pipeline = [
            {"$match": {"type.contents": {"$in": type_contents_condition}}},
        ]

        if filter_on_tx_hashes:
            pipeline.extend([{"$match": {"_id": {"$in": list_of_tx_hashes}}}])

        pipeline.extend(
            [
                {
                    "$addFields": {
                        "amount_ccd": {
                            "$first": {
                                "$map": {
                                    "input": "$result.events",
                                    "as": "events",
                                    "in": {
                                        "$trunc": {
                                            "$divide": [
                                                {"$toDouble": "$$events.amount"},
                                                1000000,
                                            ]
                                        }
                                    },
                                }
                            }
                        }
                    }
                },
                {"$match": {"amount_ccd": {"$gte": gte}}},
                {"$match": {"amount_ccd": {"$lte": lte}}},
                {"$match": {"blockSlotTime": {"$gte": start_date, "$lt": end_date}}},
                sort_condition,
                {
                    "$facet": {
                        "metadata": [{"$count": "total"}],
                        "data": [{"$skip": int(skip)}, {"$limit": int(limit)}],
                    }
                },
                {
                    "$project": {
                        "data": 1,
                        "total": {"$arrayElemAt": ["$metadata.total", 0]},
                    }
                },
            ]
        )

        return pipeline

    def search_txs_hashes_for_account_as_sender(self, account_id):
        pipeline = [{"$match": {"sender": {"$eq": account_id}}}]
        return pipeline

    def search_txs_hashes_for_account_as_receiver(self, account_id):
        pipeline = [{"$match": {"receiver": {"$eq": account_id}}}]
        return pipeline

    def search_txs_hashes_for_account_as_sender_with_params(
        self, account_id, start_block, end_block
    ):
        pipeline = [
            {"$match": {"sender_canonical": {"$eq": account_id[:29]}}},
            {"$match": {"block_height": {"$gt": start_block, "$lte": end_block}}},
        ]
        return pipeline

    def search_txs_hashes_for_account_as_receiver_with_params(
        self, account_id, start_block, end_block
    ):
        pipeline = [
            {"$match": {"receiver_canonical": {"$eq": account_id[:29]}}},
            {"$match": {"block_height": {"$gt": start_block, "$lte": end_block}}},
        ]
        return pipeline

    def search_txs_hashes_for_account_as_sender_with_params_id_only(
        self, account_id, start_block, end_block
    ):
        pipeline = [
            {"$match": {"sender_canonical": {"$eq": account_id[:29]}}},
            {"$match": {"block_height": {"$gt": start_block, "$lte": end_block}}},
            {"$project": {"_id": 1}},
        ]
        return pipeline

    def search_txs_hashes_for_account_as_receiver_with_params_id_only(
        self, account_id, start_block, end_block
    ):
        pipeline = [
            {"$match": {"receiver_canonical": {"$eq": account_id[:29]}}},
            {"$match": {"block_height": {"$gt": start_block, "$lte": end_block}}},
            {"$project": {"_id": 1}},
        ]
        return pipeline

    def search_txs_in_hash_list(self, tx_hashes):
        pipeline = [
            {"$match": {"_id": {"$in": tx_hashes}}},
            {"$sort": {"block_info.slot_time": DESCENDING}},
        ]
        return pipeline

    def search_txs_in_transactions(self, start_block, end_block):
        pipeline = [
            {
                "$match": {
                    "block_info.block_height": {"$gt": start_block, "$lte": end_block}
                }
            },
        ]
        return pipeline

    def heights_with_transactions_after(self, start_block):
        pipeline = [
            {"$match": {"block_height": {"$gt": start_block}}},
            {"$project": {"_id": 0, "blockHeight": 1}},
        ]
        return pipeline

    def search_txs_non_finalized(self):
        pipeline = [
            {"$match": {"block_info.finalized": False}},
        ]
        return pipeline

    ### exchange search
    def search_exchange_txs_as_receiver(
        self, exchanges_canonical: list, start_block: int, end_block: int
    ):
        pipeline = [
            {"$match": {"receiver_canonical": {"$in": exchanges_canonical}}},
            {"$match": {"block_height": {"$gt": start_block, "$lte": end_block}}},
        ]
        return pipeline

    def search_exchange_txs_as_sender(
        self, exchanges_canonical: list, start_block: int, end_block: int
    ):
        pipeline = [
            {"$match": {"sender_canonical": {"$in": exchanges_canonical}}},
            {"$match": {"block_height": {"$gt": start_block, "$lte": end_block}}},
        ]
        return pipeline



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: kernel.proto
# Protobuf Python Version: 4.25.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0ckernel.proto\x12\rconcordium.v2\"\x1f\n\x0e\x41\x63\x63ountAddress\x12\r\n\x05value\x18\x01 \x01(\x0c\"\x15\n\x04Memo\x12\r\n\x05value\x18\x01 \x01(\x0c\x42\x35\n\x16\x63om.concordium.grpc.v2P\x01Z\x04./pb\xaa\x02\x12\x43oncordium.Grpc.V2b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'kernel_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\026com.concordium.grpc.v2P\001Z\004./pb\252\002\022Concordium.Grpc.V2'
  _globals['_ACCOUNTADDRESS']._serialized_start=31
  _globals['_ACCOUNTADDRESS']._serialized_end=62
  _globals['_MEMO']._serialized_start=64
  _globals['_MEMO']._serialized_end=85
# @@protoc_insertion_point(module_scope)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: protocol-level-tokens.proto
# plugin: python-betterproto
# This file has been @generated

from dataclasses import dataclass
from typing import (
    List,
    Optional,
)

import betterproto

from ... import v2 as __v2__


@dataclass(eq=False, repr=False)
class Cbor(betterproto.Message):
    """A CBOR encoded bytestring"""

    value: bytes = betterproto.bytes_field(1)
    """A CBOR encoded byte string."""


@dataclass(eq=False, repr=False)
class TokenId(betterproto.Message):
    """Token ID: a unique symbol and identifier of a protocol level token."""

    value: str = betterproto.string_field(1)
    """
    Unique identifier for the token, guaranteed to be distinct across the
    entire concordium blockchain. The token id must be between 1 and 128 bytes
    long and consist of only a-z, A-Z, 0-9, `.`, `%` and `-`.
    """


@dataclass(eq=False, repr=False)
class TokenModuleRef(betterproto.Message):
    """A token module reference. This is always 32 bytes long."""

    value: bytes = betterproto.bytes_field(1)


@dataclass(eq=False, repr=False)
class TokenAmount(betterproto.Message):
    """
    PLT amount representation. The actual amount is computed as `value *
    10^(-decimals)`.
    """

    value: int = betterproto.uint64_field(1)
    """The digits of the amount."""

    decimals: int = betterproto.uint32_field(2)
    """Number of decimals in the representation"""


@dataclass(eq=False, repr=False)
class TokenState(betterproto.Message):
    """Token state at the block level"""

    token_module_ref: "TokenModuleRef" = betterproto.message_field(1)
    """The reference of the module implementing this token."""

    decimals: int = betterproto.uint32_field(3)
    """Number of decimals in the decimal number representation of amounts."""

    total_supply: "TokenAmount" = betterproto.message_field(4)
    """The total available token supply."""

    module_state: "Cbor" = betterproto.message_field(5)
    """
    Token module specific state, such as token name, feature flags, meta data.
    """


@dataclass(eq=False, repr=False)
class TokenAccountState(betterproto.Message):
    """Token state at the account level"""

    balance: "TokenAmount" = betterproto.message_field(1)
    """The available balance."""

    module_state: Optional["Cbor"] = betterproto.message_field(
        4, optional=True, group="_module_state"
    )
    """
    Token module specific account state, such as whether the account is on the
    allow or deny list.
    """


@dataclass(eq=False, repr=False)
class TokenModuleEvent(betterproto.Message):
    """
    Single token event originating from a token module as part of a token
    transaction.
    """

    type: str = betterproto.string_field(1)
    """The type of the event."""

    details: "Cbor" = betterproto.message_field(2)
    """The CBOR encoded event details."""


@dataclass(eq=False, repr=False)
class TokenHolder(betterproto.Message):
    """
    A token holder is an entity that can hold tokens. Currently, this is
    limited to accounts, but in the future it may be extended to other
    entities.
    """

    account: "__v2__.AccountAddress" = betterproto.message_field(1, group="address")
    """The account address of the holder."""


@dataclass(eq=False, repr=False)
class TokenTransferEvent(betterproto.Message):
    """
    An event emitted when a transfer of tokens from `from` to `to` is
    performed.
    """

    from_: "TokenHolder" = betterproto.message_field(1)
    """The token holder from which the tokens are transferred."""

    to: "TokenHolder" = betterproto.message_field(2)
    """The token holder to which the tokens are transferred."""

    amount: "TokenAmount" = betterproto.message_field(3)
    """The amount of tokens transferred."""

    memo: Optional["__v2__.Memo"] = betterproto.message_field(
        4, optional=True, group="_memo"
    )
    """
    An optional memo field that can be used to attach a message to the token
    transfer.
    """


@dataclass(eq=False, repr=False)
class TokenSupplyUpdateEvent(betterproto.Message):
    """
    An event emitted when the token supply is updated, i.e. by minting/burning
    tokens to/from the balance of the `target`.
    """

    target: "TokenHolder" = betterproto.message_field(1)
    """The token holder the balance update is performed on."""

    amount: "TokenAmount" = betterproto.message_field(2)
    """The balance difference to be applied to the target."""


@dataclass(eq=False, repr=False)
class TokenEvent(betterproto.Message):
    """Token event originating from token transactions."""

    token_id: "TokenId" = betterproto.message_field(1)
    """The Token ID."""

    module_event: "TokenModuleEvent" = betterproto.message_field(2, group="event")
    """An event emitted by the token module."""

    transfer_event: "TokenTransferEvent" = betterproto.message_field(3, group="event")
    """An event emitted when a transfer of tokens is performed."""

    mint_event: "TokenSupplyUpdateEvent" = betterproto.message_field(4, group="event")
    """
    An event emitted when the token supply is updated by minting tokens to a
    token holder.
    """

    burn_event: "TokenSupplyUpdateEvent" = betterproto.message_field(5, group="event")
    """
    An event emitted when the token supply is updated by burning tokens from
    the balance of a token holder.
    """


@dataclass(eq=False, repr=False)
class TokenEffect(betterproto.Message):
    """Token events originating from token transactions."""

    events: List["TokenEvent"] = betterproto.message_field(1)
    """Events emitted by the token."""


@dataclass(eq=False, repr=False)
class TokenModuleRejectReason(betterproto.Message):
    """
    Details provided by the token module in the event of rejecting a
    transaction.
    """

    token_id: "TokenId" = betterproto.message_field(1)
    """The Token ID of the protocol-level token."""

    type: str = betterproto.string_field(2)
    """The type of the reject reason."""

    details: Optional["Cbor"] = betterproto.message_field(
        3, optional=True, group="_details"
    )
    """(Optional) CBOR-encoded details."""


@dataclass(eq=False, repr=False)
class CreatePlt(betterproto.Message):
    """Update payload for creating a new protocol-level token"""

    token_id: "TokenId" = betterproto.message_field(1)
    """The Token ID of the protocol-level token."""

    token_module: "TokenModuleRef" = betterproto.message_field(2)
    """The hash that identifies the token module implementation."""

    decimals: int = betterproto.uint32_field(4)
    """
    The number of decimal places used in the representation of amounts of this
    token. This determines the smallest representable fraction of the token.
    This can be at most 255.
    """

    initialization_parameters: "Cbor" = betterproto.message_field(5)
    """The initialization parameters of the token, encoded in CBOR."""


@dataclass(eq=False, repr=False)
class TokenCreationDetails(betterproto.Message):
    """Details about the creation of a protocol-level token."""

    create_plt: "CreatePlt" = betterproto.message_field(1)
    """The update payload used to create the token."""

    events: List["TokenEvent"] = betterproto.message_field(2)
    """
    The events generated by the token module during the creation of the token.
    """



import kernel_pb2 as _kernel_pb2
import types_pb2 as _types_pb2
import protocol_level_tokens_pb2 as _protocol_level_tokens_pb2
from google.protobuf import descriptor as _descriptor
from typing import ClassVar as _ClassVar

DESCRIPTOR: _descriptor.FileDescriptor



import requests
from rich.console import Console
from ccdexplorer_fundamentals.enums import NET

console = Console()


class Mixin:
    def ql_request_tx_from_hash(self, txHash: str, net=NET.MAINNET):
        query = "query {"
        query += f'transactionByTransactionHash(transactionHash: "{txHash}")'
        query += "{"

        query += self.standard_tx_fields()
        query += self.ql_query_tx_events()

        query += "}"
        try:
            url_to_use = (
                self.graphql_url if net == NET.MAINNET else self.graphql_url_testnet
            )

            r = requests.post(url_to_use, json={"query": query})
            if r.status_code == 200:
                return r.json()["data"]["transactionByTransactionHash"]

        except Exception as e:
            console.log(query, e)
            return None



# import ssl
# from OpenSSL import crypto
# import grpc
# import os

# ssl_context = ssl.create_default_context()
# certs_der = ssl_context.get_ca_certs(binary_form=True)
# certs_x509 = [crypto.load_certificate(crypto.FILETYPE_ASN1, x) for x in certs_der]
# certs_pem = [crypto.dump_certificate(crypto.FILETYPE_PEM, x) for x in certs_x509]
# certs_bytes = b"".join(certs_pem)

# cred = grpc.ssl_channel_credentials(certs_bytes)


# # configure this dict for your systems
# system_certs_map = {
#     "Windows": "<Path to system cert>",
#     "Darwin": "/opt/homebrew/etc/ca-certificates/cert.pem",
#     "Linux": "/etc/ssl/certs/ca-bundle.crt",
# }

# os.environ["GRPC_DEFAULT_SSL_ROOTS_FILE_PATH"] = system_certs_map["Darwin"]
# channel_credentials = grpc.ssl_channel_credentials()

# channel = grpc.secure_channel(
#     "grpc.devnet-plt-alpha.concordium.com:20000", channel_credentials
# )
# try:
#     grpc.channel_ready_future(channel).result(timeout=1)

# except grpc.FutureTimeoutError:
#     print("Channel not ready within timeout period.")



# ruff: noqa: F403, F405, E402
from __future__ import annotations
from ccdexplorer_fundamentals.GRPCClient.types_pb2 import *
from ccdexplorer_fundamentals.enums import NET
from enum import Enum
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ccdexplorer_fundamentals.GRPCClient import GRPCClient
from ccdexplorer_fundamentals.GRPCClient.queries._SharedConverters import (
    Mixin as _SharedConverters,
)
import os
import sys

sys.path.append(os.path.dirname("ccdexplorer_fundamentals"))
from ccdexplorer_fundamentals.GRPCClient.CCD_Types import *


class Mixin(_SharedConverters):
    def get_winning_bakers_epoch(
        self: GRPCClient,  # type: ignore
        genesis_index: int,
        epoch: int,
        net: Enum = NET.MAINNET,
    ) -> list[CCD_WinningBaker]:
        epoch_request = self.generate_epoch_request_from_genesis(
            genesis_index=genesis_index, epoch=epoch
        )

        grpc_return_value: list[CCD_WinningBaker] | None = self.stub_on_net(
            net, "GetWinningBakersEpoch", epoch_request
        )

        if grpc_return_value is None:
            return []

        result = []
        for winner in list(grpc_return_value):
            result.append(self.convertWinningBaker(winner))

        return result
